package implement

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/dao"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	zhios_order_relate_logx "code.fnuoos.com/EggPlanet/egg_models.git/utils/logx"
	"xorm.io/xorm"
)

func NewUserRelateDb(engine *xorm.Engine) dao.UserRelateDao {
	return &UserRelateDb{Db: engine}
}

type UserRelateDb struct {
	Db *xorm.Engine
}

func (u UserRelateDb) UserRelatesByPuid(puid interface{}, limit, start int) (*[]model.UserRelate, error) {
	var m []model.UserRelate
	if limit == 0 && start == 0 {
		if err := u.Db.Where("parent_uid = ?", puid).
			Cols(`id,parent_uid,uid,level,invite_time`).
			Find(&m); err != nil {
			return nil, zhios_order_relate_logx.Warn(err)
		}
		return &m, nil
	}
	if err := u.Db.Where("parent_uid = ?", puid).
		Cols(`id,parent_uid,uid,level,invite_time`).Limit(limit, start).
		Find(&m); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}

	return &m, nil
}

func (u UserRelateDb) UserRelatesByPuIDAndLv(puid, lv interface{}) (*[]model.UserRelate, error) {
	var m []model.UserRelate
	if err := u.Db.Where("parent_uid = ? AND level = ?", puid, lv).
		Cols(`id,parent_uid,uid,level,invite_time`).
		Find(&m); err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return &m, nil
}

func (u UserRelateDb) UserRelateInsert(userRelate *model.UserRelate) (int64, error) {
	affected, err := u.Db.Insert(userRelate)
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func (u UserRelateDb) FindUserRelate(uid int64, level int) (*[]model.UserRelate, error) {
	var userRelate []model.UserRelate
	sess := u.Db.Where("uid = ?", uid)
	if level > 0 {
		sess.And("level<=?", level)
	}
	if err := sess.Asc("level").Find(&userRelate); err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if len(userRelate) == 0 {
		return nil, nil
	}
	return &userRelate, nil
}

func (u UserRelateDb) GetUserParentUserRelate(uid int64) (*model.UserRelate, error) {
	var data model.UserRelate
	get, err := u.Db.Where("level=1 and uid =?", uid).Get(&data)
	if get == false || err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	return &data, nil
}

func (u UserRelateDb) FindUserRelateByParentUid(uid int64, level int) (*[]model.UserRelate, error) {
	var userRelate []model.UserRelate
	sess := u.Db.Where("parent_uid = ?", uid)
	if level > 0 {
		sess.And("level<=?", level)
	}
	if err := sess.Asc("level").Find(&userRelate); err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if len(userRelate) == 0 {
		return nil, nil
	}
	return &userRelate, nil
}

func (u UserRelateDb) SumUserRelateByParentUid(parentUid string) (total int64, userRelate []*model.UserRelate, err error) {
	total, err = u.Db.Where("parent_uid = ?", parentUid).And("level = 1").FindAndCount(&userRelate)
	return
}

func (u UserRelateDb) PageFindUserRelateByParentUid(uid int64, page, pageSize int) (*[]model.UserRelate, int64, error) {
	var userRelate []model.UserRelate
	sess := u.Db.Where("parent_uid = ?", uid)
	count, err := sess.Limit(pageSize, (page-1)*pageSize).Asc("level").FindAndCount(&userRelate)
	if err != nil {
		return nil, 0, zhios_order_relate_logx.Error(err)
	}
	return &userRelate, count, nil
}

func (u UserRelateDb) PageFindUserRelateByParentUidAndLevel(uid int64, page, limit int, level int) (*[]model.UserRelate, int64, error) {
	var userRelate []model.UserRelate
	sess := u.Db.Where("parent_uid = ?", uid)
	if level == 1 || level == 2 {
		sess.And("level=?", level)
	} else if level == 0 {

	} else {
		sess.And("level != 1").And("level != 2")
	}
	total, err := sess.Limit(limit, (page-1)*page).Asc("uid").FindAndCount(&userRelate)
	if err != nil {
		return nil, 0, zhios_order_relate_logx.Error(err)
	}
	return &userRelate, total, nil
}

func (u UserRelateDb) UpdateUserRelateBySession(session *xorm.Session, uid int64, relate *model.UserRelate, forceColumns ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColumns != nil {
		affected, err = session.Where("uid=?", uid).MustCols(forceColumns...).Update(relate)
	} else {
		affected, err = session.Where("uid=?", uid).Update(relate)
	}
	if err != nil {
		return 0, zhios_order_relate_logx.Error(err.Error())
	}
	return affected, nil
}

func (u UserRelateDb) UserRelatesCountByParentUid(uid int64, level int) (int64, error) {
	var userRelate model.UserRelate
	var total int64
	var err error
	sess := u.Db.Where("parent_uid = ?", uid)
	if level > 0 {
		sess.And("level<=?", level)
	}
	if total, err = sess.Asc("level").Count(&userRelate); err != nil {
		return 0, zhios_order_relate_logx.Error(err)
	}
	return total, nil
}
